==> ilitools-1.1.6-2.1-any.pkg.tar.zst: usr/lib/python3.12/site-packages/ilitools/filesystem.py <==
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# Copyright (c) 2019-2023, AllWorldIT
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""IDMS Linux installer filesystem class."""

import os
import pathlib
from typing import List, Optional

from . import util
from .exceptions import IliException


class IliFilesystem:
    """IDMS Linux Installer filesystem."""

    _source: str
    _fstype: str
    _mount_root: str  # Root path of the mount
    _mount_path: str  # Mount path within the root path
    _mkfs_options: List[str]
    _mount_options: List[str]
    _uuid: Optional[str]
    _mounted: bool

    # pylint: disable=too-many-arguments
    def __init__(self, source: str, fstype: str, mount_root: str, mount_path: str, options: Optional[List[str]] = None):
        """Initialize object."""
        self._source = source
        self._fstype = fstype
        self._mount_root = mount_root
        # We treat / specially
        if mount_path == "/":
            self._mount_path = ""
        else:
            self._mount_path = mount_path
        self._mounted = False

        if options:
            self._mkfs_options = options
        else:
            self._mkfs_options = []

        self._mount_options = []

        self._uuid = None

    def mkfs(self) -> None:
        """Make filesystem and return UUID."""

        if "bind" in self.fstype:
            raise RuntimeError("Cannot mkfs on a bind mount")

        # Only add mkfs_options if its set
        args: List[str] = ["/usr/bin/mkfs", "--type", self.fstype]
        if self.mkfs_options:
            args.extend(self.mkfs_options)
        args.append(self.source)

        # Make filesystem
        util.run_host(args)

        # Run blkid on the device
        blkid_output = util.run_host(["/usr/bin/blkid", "--output", "export", self.source])
        uuid = ""
        # Loop with the lines returned
        for line in blkid_output:
            # Split on =
            key, value = line.split("=")
            # Check if we found the UUID
            if key == "UUID":
                # If we did set it and break the loop
                uuid = value
                break
        if not uuid:
            raise IliException(f"Failed to get UUID from '{self.source}'")
        # Set this filesystem UUID
        self._uuid = uuid

        # For btrfs things are a bit different, we need to temporarily mount the filesystem to create the subvolumes
        if self.fstype == "btrfs":
            # Mount filesystem
            self.mount()
            # Create subvolumes for @ and @home
            root_subvol = pathlib.Path(self.mount_point, "@")
            try:
                util.run_host(["btrfs", "subvolume", "create", str(root_subvol)])
            except OSError as err:
                raise IliException(f"Failed to create root subvolume '{root_subvol}': {err}") from None

            # Unmount filesystem
            self.unmount()
            # Add subvol options
            self.mount_options.insert(0, "subvol=@")

    def mount(self, extra_mount_options: Optional[List[str]] = None) -> None:
        """Mount src to dst."""

        mount_args = []
        if self.fstype == "bind":
            mount_args.append("--bind")
        else:
            mount_args.extend(["-t", self.fstype])

        # Work out mount options to pass
        mount_options: List[str] = []
        if self.mount_options:
            mount_options.extend(self.mount_options)
        if extra_mount_options:
            mount_options.extend(extra_mount_options)
        if mount_options:
            mount_args.extend(["-o", ",".join(mount_options)])

        # Check if path exists and if it does, if it is a directory
        if os.path.exists(self.mount_point) and not os.path.isdir(self.mount_point):
            raise IliException(f"Mount point '{self.mount_point}' is not a directory")

        # Create mount point
        try:
            pathlib.Path(self.mount_point).mkdir(mode=0o0755, parents=True, exist_ok=True)
        except OSError as err:
            raise IliException(f"Failed to create boot directory '{self.mount_point}': {err}") from None

        util.run_host(["mount", self.source, self.mount_point, *mount_args])
        # Flag filesystem as mounted
        self._mounted = True

    def unmount(self) -> None:
        """Unmount a path."""

        if not self.mounted:
            raise RuntimeError(f"Cannot unmount an unmounted filesystem {self.source}")

        # Unmount path
        util.run_host(["/usr/bin/umount", self.mount_point])
        # Flag filesystem as no longer mounted
        self._mounted = False

    @property
    def source(self) -> str:
        """Return source."""
        return self._source

    @property
    def fstype(self) -> str:
        """Return filesystem type."""
        return self._fstype

    @property
    def mount_root(self) -> str:
        """Return mount root."""
        return self._mount_root

    @property
    def mount_path(self) -> str:
        """Return mount path."""
        return self._mount_path

    @property
    def mount_point(self) -> str:
        """Return mount point which is based on the root + path."""
        return f"{self._mount_root}{self._mount_path}"

    @property
    def mkfs_options(self) -> Optional[List[str]]:
        """Return mkfs options."""
        return self._mkfs_options

    @property
    def mount_options(self) -> List[str]:
        """Return mount options."""
        return self._mount_options

    @property
    def mounted(self) -> bool:
        """Return a bool indicating if this filesystem is mounted or not."""
        return self._mounted

    @property
    def uuid(self) -> Optional[str]:
        """Return UUID of filesystem."""
        return self._uuid

==> ilitools-1.2.4-1.1-any.pkg.tar.zst: usr/lib/python3.12/site-packages/ilitools/filesystem.py <==
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# Copyright (c) 2019-2024, AllWorldIT
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""IDMS Linux installer filesystem class."""

import os
import pathlib
from typing import List, Optional

from . import util
from .exceptions import IliException

__all__ = ["IliFilesystem"]


class IliFilesystem:  # pylint: disable=too-many-instance-attributes
    """IDMS Linux Installer filesystem."""

    _source: str
    _fstype: str
    _mount_root: str  # Root path of the mount
    _mount_path: str  # Mount path within the root path
    _mkfs_options: List[str]
    _mount_options: List[str]
    _uuid: Optional[str]
    _mounted: bool

    def __init__(  # pylint: disable=too-many-arguments,too-many-positional-arguments
        self, source: str, fstype: str, mount_root: str, mount_path: str, options: Optional[List[str]] = None
    ):
        """Initialize object."""
        self._source = source
        self._fstype = fstype
        self._mount_root = mount_root
        # We treat / specially
        if mount_path == "/":
            self._mount_path = ""
        else:
            self._mount_path = mount_path
        self._mounted = False

        if options:
            self._mkfs_options = options
        else:
            self._mkfs_options = []

        self._mount_options = []

        self._uuid = None

    def mkfs(self) -> None:
        """Make filesystem and return UUID."""

        if "bind" in self.fstype:
            raise RuntimeError("Cannot mkfs on a bind mount")

        # Only add mkfs_options if its set
        args: List[str] = ["/usr/bin/mkfs", "--type", self.fstype]
        if self.mkfs_options:
            args.extend(self.mkfs_options)
        args.append(self.source)

        # Make filesystem
        util.run_host(args)

        # Run blkid on the device
        blkid_output = util.run_host(["/usr/bin/blkid", "--output", "export", self.source])
        uuid = ""
        # Loop with the lines returned
        for line in blkid_output:
            # Split on =
            key, value = line.split("=")
            # Check if we found the UUID
            if key == "UUID":
                # If we did set it and break the loop
                uuid = value
                break
        if not uuid:
            raise IliException(f"Failed to get UUID from '{self.source}'")
        # Set this filesystem UUID
        self._uuid = uuid

        # For btrfs things are a bit different, we need to temporarily mount the filesystem to create the subvolumes
        if self.fstype == "btrfs":
            # Mount filesystem
            self.mount()
            # Create subvolumes for @ and @home
            root_subvol = pathlib.Path(self.mount_point, "@")
            try:
                util.run_host(["btrfs", "subvolume", "create", str(root_subvol)])
            except OSError as err:
                raise IliException(f"Failed to create root subvolume '{root_subvol}': {err}") from None

            # Unmount filesystem
            self.unmount()
            # Add subvol options
            self.mount_options.insert(0, "subvol=@")

    def mount(self, extra_mount_options: Optional[List[str]] = None) -> None:
        """Mount src to dst."""

        mount_args = []
        if self.fstype == "bind":
            mount_args.append("--bind")
        else:
            mount_args.extend(["-t", self.fstype])

        # Work out mount options to pass
        mount_options: List[str] = []
        if self.mount_options:
            mount_options.extend(self.mount_options)
        if extra_mount_options:
            mount_options.extend(extra_mount_options)
        if mount_options:
            mount_args.extend(["-o", ",".join(mount_options)])

        # Check if path exists and if it does, if it is a directory
        if os.path.exists(self.mount_point) and not os.path.isdir(self.mount_point):
            raise IliException(f"Mount point '{self.mount_point}' is not a directory")

        # Create mount point
        try:
            pathlib.Path(self.mount_point).mkdir(mode=0o0755, parents=True, exist_ok=True)
        except OSError as err:
            raise IliException(f"Failed to create boot directory '{self.mount_point}': {err}") from None

        util.run_host(["mount", self.source, self.mount_point, *mount_args])
        # Flag filesystem as mounted
        self._mounted = True

    def unmount(self) -> None:
        """Unmount a path."""

        if not self.mounted:
            raise RuntimeError(f"Cannot unmount an unmounted filesystem {self.source}")

        # Unmount path
        util.run_host(["/usr/bin/umount", self.mount_point])
        # Flag filesystem as no longer mounted
        self._mounted = False

    @property
    def source(self) -> str:
        """Return source."""
        return self._source

    @property
    def fstype(self) -> str:
        """Return filesystem type."""
        return self._fstype

    @property
    def mount_root(self) -> str:
        """Return mount root."""
        return self._mount_root

    @property
    def mount_path(self) -> str:
        """Return mount path."""
        return self._mount_path

    @property
    def mount_point(self) -> str:
        """Return mount point which is based on the root + path."""
        return f"{self._mount_root}{self._mount_path}"

    @property
    def mkfs_options(self) -> Optional[List[str]]:
        """Return mkfs options."""
        return self._mkfs_options

    @property
    def mount_options(self) -> List[str]:
        """Return mount options."""
        return self._mount_options

    @property
    def mounted(self) -> bool:
        """Return a bool indicating if this filesystem is mounted or not."""
        return self._mounted

    @property
    def uuid(self) -> Optional[str]:
        """Return UUID of filesystem."""
        return self._uuid
